import { NS } from "@ns";
import { getControllerState, setControllerState } from "/lib/types-and-constants";

/** @param {NS} ns */
export async function main(ns : NS) : Promise<void> {
  const datafileState = getControllerState(ns)

  ns.print("INFO Starting automated scan . . .\n")
  let foundServers : string[] = []
  let serversToScan = datafileState.knownServers.map(server => server.hostname)

  if(!serversToScan.length) {
    serversToScan = ['home']
  }

  while (serversToScan.length) {
    const currentServer = serversToScan.shift() as string

    if(foundServers.includes(currentServer)) continue

    foundServers = [...foundServers, currentServer]

    ns.print(`INFO Scanning from ${currentServer}\n`)
    const newVisibleServers = ns.scan(currentServer)
      .filter(server => !foundServers.includes(server))
    serversToScan = [...serversToScan, ...newVisibleServers]
    if(newVisibleServers.length > 0)
      ns.print(`SUCCESS - Discovered ${newVisibleServers.length} new servers to add to scan list\n`)
  }

  datafileState.knownServers = foundServers.map(ns.getServer)
  setControllerState(ns, datafileState)
}

