import { NS } from "@ns";
import { getControllerState, setControllerState } from "/lib/types-and-constants";

export async function main(ns : NS) : Promise<void> {
  const datafileState = getControllerState(ns)
  const {knownServers} = datafileState

  const portConfigs = [{
    openProperty: 'sshPortOpen',
    program: 'BruteSSH.exe',
    func: ns.brutessh
  }, {
    openProperty: 'ftpPortOpen',
    program: 'FTPCrack.exe',
    func: ns.ftpcrack
  }, {
    openProperty: 'sqlPortOpen',
    program: 'SQLInject.exe',
    func: ns.sqlinject
  }, {
    openProperty: 'httpPortOpen',
    program: 'HTTPWorm.exe',
    func: ns.httpworm
  }, {
    openProperty: 'smtpPortOpen',
    program: 'relaySMTP.exe',
    func: ns.relaysmtp
  }]

  for (const server of knownServers) {
    if (server.hostname === 'home' || server.hasAdminRights)
      continue

    const requiredPorts = server.numOpenPortsRequired ?? 0
    const alreadyOpen = server.openPortCount ?? 0
    const openablePorts = portConfigs.filter(port =>
      !(server as any)[port.openProperty]
      && ns.fileExists(port.program))
    const numberCanOpen = openablePorts.length

    if (alreadyOpen + numberCanOpen < requiredPorts) {
      ns.print(`WARN: Unable to nuke ${server.hostname}, cannot open enough ports (${alreadyOpen} + ${numberCanOpen} < ${requiredPorts})`)
      continue
    }

    for (const port of openablePorts) {
      ns.tprint(`INFO: Opening ${port.openProperty.replace('PortOpen', '')} on ${server.hostname}`)
      port.func(server.hostname);
      (server as any)[port.openProperty] = true
    }

    ns.tprint(`SUCCESS: Nuking ${server.hostname}`)
    ns.nuke(server.hostname)

    server.hasAdminRights = true
  }

  setControllerState(ns, {
    ...datafileState,
    knownServers
  })
}
